@@ -2,7 +2,7 @@
  * nvidia-persistenced: A daemon for maintaining persistent driver state,
  * specifically for use by the NVIDIA Linux driver.
  *
- * Copyright (C) 2013-2016 NVIDIA Corporation
+ * Copyright (C) 2013-2018 NVIDIA Corporation
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -26,6 +26,7 @@
  */
 
 #include <errno.h>
+#include <grp.h>
 #include <pwd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -42,22 +43,20 @@
 #define TAB    "  "
 #define BIGTAB "      "
 
-extern const char *pNV_ID;
-
 /*
  * print_version() - print basic version information about the utility.
  */
 static void print_version(void)
 {
     nv_info_msg(NULL, "");
-    nv_info_msg(NULL, "%s", pNV_ID);
+    nv_info_msg(NULL, "%s", NV_ID_STRING);
     nv_info_msg(NULL, "");
     nv_info_msg(TAB, "The NVIDIA Persistence Daemon.");
     nv_info_msg(NULL, "");
     nv_info_msg(TAB, "A tool for maintaining persistent driver state, "
                      "specifically for use by the NVIDIA Linux driver.");
     nv_info_msg(NULL, "");
-    nv_info_msg(TAB, "Copyright (C) 2013-2016 NVIDIA Corporation.");
+    nv_info_msg(TAB, "Copyright (C) 2013-2018 NVIDIA Corporation.");
     nv_info_msg(NULL, "");
 }
 
@@ -99,6 +98,7 @@ static void print_help(void)
 static void setup_option_defaults(NvPdOptions *options)
 {
     options->persistence_mode = NV_PERSISTENCE_MODE_ENABLED;
+    options->uvm_persistence_mode = NV_UVM_PERSISTENCE_MODE_DISABLED;
     options->nvidia_cfg_path = NULL;
     options->verbose = 0;
     options->uid = getuid();
@@ -115,6 +115,8 @@ void parse_options(int argc, char *argv[], NvPdOptions *options)
     int boolval;
     char *strval;
     struct passwd *pw_entry;
+    struct group *gr_entry;
+    int group_specified = 0;
 
     setup_option_defaults(options);
 
@@ -145,12 +147,25 @@ void parse_options(int argc, char *argv[], NvPdOptions *options)
             case 'u':
                 pw_entry = getpwnam(strval);
                 if (pw_entry == NULL) {
-                    nv_error_msg("Failed to find user ID of user '%s': %s", strval,
-                                 strerror(errno));
+                    nv_error_msg("Failed to find user ID of user '%s': %s",
+                                 strval, strerror(errno));
                     exit(EXIT_FAILURE);
                 }
                 options->uid = pw_entry->pw_uid;
-                options->gid = pw_entry->pw_gid;
+                if (!group_specified) {
+                    options->gid = pw_entry->pw_gid;
+                }
+                break;
+            case 'g':
+                gr_entry = getgrnam(strval);
+                if (gr_entry == NULL) {
+                    nv_error_msg("Failed to find group ID of group '%s': %s",
+                                 strval, strerror(errno));
+                    exit(EXIT_FAILURE);
+                }
+                /* overrides the gid from the -u option, if already given */
+                options->gid = gr_entry->gr_gid;
+                group_specified = 1;
                 break;
             case PERSISTENCE_MODE_OPTION:
                 if (boolval) {
@@ -159,6 +174,13 @@ void parse_options(int argc, char *argv[], NvPdOptions *options)
                     options->persistence_mode = NV_PERSISTENCE_MODE_DISABLED;
                 }
                 break;
+            case UVM_PERSISTENCE_MODE_OPTION:
+                if (boolval) {
+                    options->uvm_persistence_mode = NV_UVM_PERSISTENCE_MODE_ENABLED;
+                } else {
+                    options->uvm_persistence_mode = NV_UVM_PERSISTENCE_MODE_DISABLED;
+                }
+                break;
             case NVIDIA_CFG_PATH_OPTION:
                 options->nvidia_cfg_path = strval;
                 break;
